import argparse
import os
import json

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids

from datastation.dv_api import get_dataset_locks, delete_dataset_locks


def unlock_dataset_action(server_url, api_token, pid):
    deleted_locks = False
    resp_data = get_dataset_locks(server_url, pid)
    # print(json.dumps(resp_data, indent=2))
    if len(resp_data) == 0:
        print("No locks")
        print("Leave as-is")
    else:
        print("Found locks")
        print(json.dumps(resp_data, indent=2))
        # delete
        print("Try deleting the locks")
        delete_dataset_locks(server_url, api_token, pid)
        print("Done")
        deleted_locks = True

    return deleted_locks


def unlock_dataset_command(config, pids_file):
    full_name = os.path.join(config['files']['output_dir'], pids_file)
    pids = load_pids(full_name)

    # could be fast, but depends on number of files inside the dataset
    batch_process(pids,
                  lambda pid: unlock_dataset_action(config['dataverse']['server_url'], config['dataverse']['api_token'],
                                                    pid), config['files']['output_dir'], delay=1.5)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Unlock datasets (if locked) with the pids in the given input file')
    parser.add_argument('-p', '--pids-file', default='dataset_pids.txt', help='The input file with the dataset pids')
    args = parser.parse_args()
    unlock_dataset_command(config, args.pids_file)


if __name__ == '__main__':
    main()



import argparse

import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_metadatafile import store_dataset_result
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import get_dataset_metadata_export


def retrieve_dataset_metadata_action(server_url, pid, output_dir):
    dataset_metadata = get_dataset_metadata_export(server_url, pid)
    # note that the dataset metadata might be large if there are a lot of files in the dataset!
    store_dataset_result(pid, dataset_metadata, output_dir)
    # store_dataset_result_as_xml(pid, dataset_metadata, save_path)


def retrieve_dataset_metadata_command(config, input_filename, output_dir):
    print('Args: ' + input_filename + ',  ' + output_dir)
    print("Example using server URL: " + config['dataverse']['server_url'])

    # create output dir if not exists!
    #work_path = os.path.dirname(CONFIG.OUTPUT_DIR)
    save_path = os.path.join(config['files']['output_dir'], output_dir)
    if os.path.isdir(save_path):
        print("Skipping dir creation, because it already exists: " + save_path)
    else:
        print("Creating output dir: " + save_path)
        os.makedirs(save_path)

    # look for inputfile in configured OUTPUT_DIR
    full_name = os.path.join(config['files']['output_dir'], input_filename)
    pids = load_pids(full_name)

    batch_process(pids, lambda pid: retrieve_dataset_metadata_action(config['dataverse']['server_url'], pid, save_path), config['files']['output_dir'], delay=0.2)



def main():
    config = init()
    parser = argparse.ArgumentParser(description='Retrieves the metadata for all published datasets with the pids in the given inputfile')
    parser.add_argument('-p', '--pids-file', default='dataset_pids.txt', help='The input file with the dataset pids')
    parser.add_argument('-o', '--output', default='dataset_metadata', help='The output dir, for storing the metadata files retrieved')
    args = parser.parse_args()

    retrieve_dataset_metadata_command(config, args.pids_file, args.output)

if __name__ == '__main__':
    main()



import argparse
import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids

from datastation.dv_api import replace_dataset_metadatafield, get_dataset_metadata


def replace_metadata_field_value_action(server_url, api_token, pid, mdb_name, field_name, field_from_value,
                                        field_to_value):
    """

    :param pid:
    :param mdb_name:
    :param field_name:
    :param field_from_value:
    :param field_to_value:
    :return:
    """
    # Getting the metadata is not always needed when doing a replace,
    # but when you need to determine if replace is needed by inspecting the current content
    # you need to 'get' it first.
    # Another approach would be to do that selection up front (via search)
    # and have that generate a list with pids to process 'blindly'.
    resp_data = get_dataset_metadata(server_url, api_token, pid)
    # print(resp_data['datasetPersistentId'])
    mdb_fields = resp_data['metadataBlocks'][mdb_name]['fields']
    # print(json.dumps(mdb_fields, indent=2))

    # metadata field replacement (an idempotent action I think)
    # replace replace_from with replace_to for field with typeName replace_field
    replace_field = field_name
    replace_from = field_from_value
    replace_to = field_to_value

    replaced = False
    for field in mdb_fields:
        # expecting (assuming) one and only one instance,
        # but the code will try to change all it can find
        if field['typeName'] == replace_field:
            print("Found " + replace_field + ": " + field['value'])
            if field['value'] == replace_from:
                updated_field = field.copy()
                # be save and mutate a copy
                updated_field['value'] = replace_to
                print("Try updating it with: " + updated_field['value'])
                replace_dataset_metadatafield(server_url, api_token, pid, updated_field)
                print("Done")
                replaced = True
            else:
                print("Leave as-is")
    return replaced


def replace_metadata_field_value_command(config, server_url, api_token, pids_file, mdb_name, field_name,
                                         field_from_value, field_to_value):
    # look for inputfile in configured OUTPUT_DIR
    full_name = os.path.join(config['files']['output_dir'], pids_file)
    pids = load_pids(full_name)

    batch_process(pids,
                  lambda pid: replace_metadata_field_value_action(server_url, api_token, pid, mdb_name, field_name,
                                                                  field_from_value,
                                                                  field_to_value), config['files']['output_dir'],
                  delay=5.0)


# Note that the datasets that got changed get into a DRAFT status
# and at some point need to be published with a minor version increment.
# This is not done here, because you might want several (other) changes
# on the same datasets before publishing.
def main():
    config = init()
    parser = argparse.ArgumentParser(
        description='Replace metadata field in datasets with the pids in the given inputfile')
    parser.add_argument("metadatablock_name", help="Name of the metadata block")
    parser.add_argument("field_name", help="Name of the field (json typeName)")
    parser.add_argument("field_from_value", help="Value to be replaced")
    parser.add_argument("field_to_value", help="Value replacing (the new value)")
    parser.add_argument('-p', '--pids_file', default='dataset_pids.txt', help='The input file with the dataset pids')
    args = parser.parse_args()

    # have a look at the json metadata export (dataverse_json) to see what names are possible
    mdb_name = 'citation'
    field_name = 'title'
    field_from_value = 'test1'
    field_to_value = 'test1-X'

    # use config for server url and api token
    server_url = config['dataverse']['server_url']
    api_token = config.DATAVERSE_API_TOKEN

    replace_metadata_field_value_command(config, server_url, api_token, args.pids_file, mdb_name, field_name,
                                         field_from_value, field_to_value)


if __name__ == '__main__':
    main()



import os
from datetime import datetime


# store in text file, that can be loaded with load_pids
def store_pids(pids, save_path, filename=None):
    if filename is None:
        # by default use a timestamp in the filename
        timestamp_str = '_' + datetime.now().strftime("%Y%m%d_%H%M%S")
        filename = 'pids' + timestamp_str + '.txt'
    full_name = os.path.join(save_path, filename)
    print('Storing: ' + full_name)
    with open(full_name, "w") as outfile:
        outfile.write("\n".join(pids))


# pids producing function (extracting from a file)
def load_pids(file_path):
    # should read from a text file, with each line contains a pid
    pids = []
    with open(file_path) as f:
        pids = f.read().splitlines()
        # Note that f.readlines() would miss the last 'line' if it has no newline character
    # trim whitespace and remove empty lines...
    return list(filter(lambda item: item.strip(), pids))



import sys
import argparse
import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import delete_dataset_draft


def delete_dataset_command(server_url, api_token, output_dir, pids_file):
    full_name = os.path.join(output_dir, pids_file)
    pids = load_pids(full_name)

    batch_process(pids, lambda pid: delete_dataset_draft(server_url, api_token, pid), output_dir, delay=2.0)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Delete datasets with the pids in the given inputfile. '
                                                 'Only the draft version is deleted '
                                                 'and it will fail if it is not a draft!')
    parser.add_argument('-p', '--pids_file', default='dataset_pids.txt', help='The input file with the dataset pids')
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    api_token = config.DATAVERSE_API_TOKEN

    print("Deleting datasets on: {}".format(server_url))
    print("Be aware that this is irreversible and you might lose information!")
    # Only proceed if user is sure
    if not input("Are you sure? (y/n): ").lower().strip()[:1] == "y": print("Cancelling deletion"), sys.exit(1)
    print("Starting deletion")

    delete_dataset_command(server_url, api_token, config['files']['output_dir'], args.pids_file)


if __name__ == '__main__':
    main()



import argparse
import csv
import hashlib
import logging
from lxml import etree
import os.path
import psycopg
import re
import requests
import shutil
import sys
from datastation.config import init

provenance_element = 'provenance ' \
                     'xmlns:prov="http://easy.dans.knaw.nl/schemas/bag/metadata/prov/" ' \
                     'xsi:schemaLocation="http://easy.dans.knaw.nl/schemas/bag/metadata/prov/ https://easy.dans.knaw.nl/schemas/bag/metadata/prov/provenance.xsd" ' \
                     'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ' \
                     'xmlns:id-type="http://easy.dans.knaw.nl/schemas/vocab/identifier-type/" ' \
                     'xmlns:dcx-gml="http://easy.dans.knaw.nl/schemas/dcx/gml/" ' \
                     'xmlns:dcx-dai="http://easy.dans.knaw.nl/schemas/dcx/dai/" ' \
                     'xmlns:dcterms="http://purl.org/dc/terms/" xmlns:dc="http://purl.org/dc/elements/1.1/" ' \
                     'xmlns:abr="http://www.den.nl/standaard/166/Archeologisch-Basisregister/" ' \
                     'xmlns:ddm="http://easy.dans.knaw.nl/schemas/md/ddm/"'

provenance_schema_doc = etree.parse('http://easy.dans.knaw.nl/schemas/bag/metadata/prov/provenance.xsd')
provenance_xmlschema = etree.XMLSchema(provenance_schema_doc)


def validate(xml_path: str) -> bool:
    try:
        xml_doc = etree.parse(xml_path)
        provenance_xmlschema.assertValid(xml_doc)
        return True
    except etree.XMLSyntaxError as e:
        # info instead of error because we expect the old-provenance to have syntax errors
        logging.info(e)
        return False
    except etree.DocumentInvalid as e:
        # info instead of error because we expect the old-provenance to be invalid
        logging.info(e)
        return False


def replace_provenance_tag(infile):
    old_provenance_tag = "<([a-zA-Z0-9_:]*)provenance .*>"
    new_str = '<\\1' + provenance_element + '>'
    replaced = False
    element_tag = ""
    line_list = []
    with open(infile) as f:
        for item in f:
            if not replaced and ("<prov:provenance" in item or len(element_tag) > 1):
                element_tag += item.strip('\n')
                if ">" in element_tag:
                    new_item = re.sub(old_provenance_tag, new_str, element_tag)
                    line_list.append(new_item)
                    replaced = True
                    element_tag = ""
            else:
                element_tag = ""
                line_list.append(item)

    with open(infile, "w") as f:
        f.truncate()
        for line in line_list:
            f.writelines(line)


def add_result(output_file, doi: str, storage_identifier: str, old_checksum: str, dvobject_id: str, status: str,
               new_checksum: str = None):
    if not new_checksum:
        new_checksum = old_checksum
    csvwriter = csv.writer(output_file)
    csvwriter.writerow([doi, storage_identifier, old_checksum, new_checksum, old_checksum is not new_checksum,
                        dvobject_id, status])
    output_file.flush()


def dv_file_validation(dv_api_token, dv_server_url, dvobject_id, dry_run_temp_file):
    if dry_run_temp_file:
        logging.info("dry run: no Dataverse file validation for {}".format(dvobject_id))
        return True

    headers = {'X-Dataverse-key': dv_api_token}
    dv_resp = requests.post(dv_server_url + '/api/admin/validateDataFileHashValue/' + dvobject_id,
                            headers=headers)
    dv_resp.raise_for_status()

    if dv_resp.json()['status'] == "OK":
        return True
    else:
        logging.error(dv_resp.json()["message"])
        return False


def calculate_checksum(file, dry_run_temp_file):
    if dry_run_temp_file:
        file_to_check = dry_run_temp_file
    else:
        file_to_check = file

    with open(file_to_check, 'rb') as f:
        new_checksum = hashlib.sha1()
        while True:
            chunk = f.read(16 * 1024)
            if not chunk:
                break
            new_checksum.update(chunk)
    return new_checksum.hexdigest()


def restore_old_provenance_file(provenance_path, dry_run_temp_file):
    if not dry_run_temp_file:
        shutil.copyfile(provenance_path + ".old-provenance", provenance_path)
        delete_old_provenance_file(provenance_path, dry_run_temp_file)


def make_new_provenance_file(provenance_path, dry_run_temp_file):
    if dry_run_temp_file:
        new_provenance_file = dry_run_temp_file
    else:
        new_provenance_file = provenance_path + ".new-provenance"
    logging.info("fixing {}".format(provenance_path))
    shutil.copyfile(provenance_path, new_provenance_file)
    replace_provenance_tag(new_provenance_file)
    return new_provenance_file


def replace_old_with_new_provenance_file(provenance_path, new_provenance_file, dry_run_temp_file):
    if not dry_run_temp_file:
        old_provenance_file = provenance_path + ".old-provenance"
        shutil.copyfile(provenance_path, old_provenance_file)
        shutil.move(new_provenance_file, provenance_path)


def update_dvndb_record(dry_run_temp_file, provenance_path, dvndb, output_file, doi, storage_identifier, old_checksum,
                        new_checksum, dvobject_id):
    update_statement = "update datafile set checksumvalue = '{}' where id = {} and checksumvalue='{}'" \
        .format(new_checksum, dvobject_id, old_checksum)
    logging.info(update_statement)
    if not dry_run_temp_file:
        with dvndb.cursor() as dvndb_cursor:
            try:
                dvndb_cursor.execute(update_statement)
                dvndb.commit()
            except psycopg.DatabaseError as error:
                logging.error(error)
                add_result(output_file, doi=doi, storage_identifier=storage_identifier, old_checksum=old_checksum,
                           new_checksum=new_checksum, dvobject_id=dvobject_id, status="FAILED")
                restore_old_provenance_file(provenance_path, dry_run_temp_file)
                sys.exit("FATAL ERROR: problem updating dvndb record for {} and file {}".format(doi, dvobject_id))


def delete_old_provenance_file(provenance_path, dry_run_temp_file):
    if not dry_run_temp_file:
        os.remove(provenance_path + ".old-provenance")


def process_dataset(file_storage_root, doi, storage_identifier, current_checksum, dvobject_id: str,
                    dvndb, dv_server_url, dv_api_token, output_file, dry_run_file):
    logging.info("processing {} with file {}".format(doi, storage_identifier))
    provenance_path = os.path.join(file_storage_root, doi, storage_identifier)
    if not os.path.exists(provenance_path):
        sys.exit("FATAL ERROR: {} does not exist for doi {}".format(provenance_path, doi))

    with open(provenance_path) as provenance_file:
        if not is_provenance_xml_file(provenance_file):
            sys.exit("FATAL ERROR: {} for doi {} is not a provenance xml file".format(provenance_file, doi))

        if validate(provenance_path):
            logging.info("SUCCESS: {} is already valid for doi {}".format(storage_identifier, doi))
            add_result(output_file, doi=doi, storage_identifier=storage_identifier,
                       old_checksum=current_checksum, dvobject_id=dvobject_id, status="OK")
            return

        new_provenance_file = make_new_provenance_file(provenance_path, dry_run_file)

        if not validate(new_provenance_file):
            add_result(output_file, doi=doi, storage_identifier=storage_identifier,
                       old_checksum=current_checksum, dvobject_id=dvobject_id, status="FAILED")
            sys.exit("FATAL ERROR: new provenance file not valid for {} at {}".format(doi, new_provenance_file))

        new_checksum = calculate_checksum(new_provenance_file, dry_run_file)
        replace_old_with_new_provenance_file(provenance_path, new_provenance_file, dry_run_file)
        update_dvndb_record(dry_run_file, provenance_path, dvndb, output_file, doi=doi,
                            storage_identifier=storage_identifier, old_checksum=current_checksum,
                            new_checksum=new_checksum, dvobject_id=dvobject_id)

        if dv_file_validation(dv_api_token, dv_server_url, dvobject_id, dry_run_file):
            logging.info("SUCCESS: {} is fixed for doi {}".format(storage_identifier, doi))
            delete_old_provenance_file(provenance_path, dry_run_file)
            add_result(output_file, doi=doi, storage_identifier=storage_identifier,
                       old_checksum=current_checksum,
                       new_checksum=new_checksum, dvobject_id=dvobject_id, status="OK")
        else:
            add_result(output_file, doi=doi, storage_identifier=storage_identifier,
                       old_checksum=current_checksum,
                       new_checksum=new_checksum, dvobject_id=dvobject_id, status="FAILED")
            # rollback the database update, resetting the original checksum
            update_dvndb_record(dry_run_file, provenance_path, dvndb, output_file, doi=doi,
                                storage_identifier=storage_identifier, old_checksum=new_checksum,
                                new_checksum=current_checksum, dvobject_id=dvobject_id)
            restore_old_provenance_file(provenance_path, dry_run_file)
            sys.exit("FATAL ERROR: Dataverse file validation failed for new provenance file with id {} of {}"
                     .format(dvobject_id, doi))


def is_provenance_xml_file(provenance_file):
    # don't parse the file as xml, just check the first line and tag
    xml_prolog_found = provenance_file.readline().rstrip() == '<?xml version="1.0" encoding="UTF-8"?>'
    if xml_prolog_found:
        line_count = 0
        for line in provenance_file:
            if "<prov:provenance" in line:
                return True
            if line_count > 2:
                return False
            line_count += 1
    return False


def connect_to_database(user: str, password: str):
    return psycopg.connect(
        "host={} dbname={} user={} password={}".format('localhost', 'dvndb', user, password))


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Fixes one or more invalid provenance.xml files. With the optional parameters, it is possible to process one dataset/provenance.xml.'
                    + ' If none of the optional parameters is provided the input-file is expected to contain a CSV file with the columns: doi, storage_identifier, current_sha1_checksum and dvobject_id')
    parser.add_argument('-d', '--doi', dest='doi', help='the dataset DOI')
    parser.add_argument('-s', '--storage-identifier', dest='storage_identifier',
                        help='the storage identifier of the provenance.xml file')
    parser.add_argument('-c', '--current-sha1-checksum', dest='current_sha1_checksum',
                        help='the expected current checksum of the provenance.xml file')
    parser.add_argument('-o', '--dvobject-id', dest='dvobject_id',
                        help='the dvobject.id for the provenance.xml in dvndb')
    parser.add_argument('-u', '--user', dest='dvndb_user', help="dvn user with update privileges on 'datafile'")
    parser.add_argument('-p', '--password', dest='dvndb_password', help="password for dvn user")
    parser.add_argument('-l', '--log', dest='output_csv', help='the csv log file with the result per doi',
                        default='fix-provenance-output.csv')
    parser.add_argument('-r', '--dryrun', dest='dryrun', help="only logs the actions, nothing is executed",
                        action='store_true')
    parser.add_argument('-i', '--input-file', dest='input_file',
                        help="csv file with columns: doi, storage_identifier, current_sha1_checksum and dvobject_id")
    args = parser.parse_args()

    dvndb_conn = None
    try:
        dry_run_provenance_file_path = None
        if args.dryrun:
            dry_run_provenance_file_path = "dry-run-provenance-file.xml"
            logging.info("--- DRY RUN, using {} for the temporary provenance file ---"
                         .format(dry_run_provenance_file_path))
        else:
            if not (args.dvndb_password and args.dvndb_user):
                sys.exit("please provide dvndb user and password when not in dry-run mode")
            dvndb_conn = connect_to_database(args.dvndb_user, args.dvndb_password)

        with open(args.output_csv, "w") as output_csv:
            logging.info("writing to " + args.output_csv)
            headers = ["doi", "storage_identifier", "old_checksum", "new_checksum", "updated", "dvobject_id", "status"]
            csv_writer = csv.DictWriter(output_csv, headers)
            csv_writer.writeheader()
            if args.input_file:
                with open(args.input_file, "r") as input_file_handler:
                    csv_reader = csv.DictReader(input_file_handler, delimiter=',')
                    for row in csv_reader:
                        process_dataset(file_storage_root=config['dataverse']['files_root'], doi=row["doi"],
                                        storage_identifier=row["storage_identifier"],
                                        current_checksum=row["current_sha1_checksum"],
                                        dvobject_id=row["dvobject_id"],
                                        dvndb=dvndb_conn, dv_server_url=config['dataverse']['server_url'],
                                        dv_api_token=config['dataverse']['api_token'],
                                        output_file=output_csv, dry_run_file=dry_run_provenance_file_path)
            else:
                process_dataset(file_storage_root=config['dataverse']['files_root'], doi=args.doi,
                                storage_identifier=args.storage_identifier,
                                current_checksum=args.current_sha1_checksum, dvobject_id=args.dvobject_id,
                                dvndb=dvndb_conn,
                                dv_server_url=config['dataverse']['server_url'],
                                dv_api_token=config['dataverse']['api_token'], output_file=output_csv,
                                dry_run_file=dry_run_provenance_file_path)
        output_csv.close()
        if not args.dryrun:
            dvndb_conn.close()
    except psycopg.DatabaseError as error:
        logging.error(error)
    finally:
        if dvndb_conn is not None:
            dvndb_conn.close()


if __name__ == '__main__':
    main()



import argparse
import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import reindex_dataset


def reindex_dataset_command(config, server_url, pids_file):
    # look for inputfile in configured OUTPUT_DIR
    full_name = os.path.join(config['files']['output_dir'], pids_file)
    pids = load_pids(full_name)

    # could be fast, but depends on number of files inside the dataset
    batch_process(pids, lambda pid: reindex_dataset(server_url, pid), config['files']['output_dir'], delay=1.5)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Reindex datasets with the pids in the given inputfile')
    parser.add_argument('-p', '--pids_file', default='dataset_pids.txt', help='The input file with the dataset pids')
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']

    reindex_dataset_command(config, server_url, args.pids_file)


if __name__ == '__main__':
    main()



import argparse
import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_metadatafile import construct_filename_base_from_pid
from datastation.ds_pidsfile import load_pids


def convert_dataset_metadata_action(server_url, pid, input_dir, output_dir):
    # find input metadata file
    input_filename_ext = 'json'  # assume json, but maybe detect this?
    input_full_name = os.path.join(input_dir, construct_filename_base_from_pid(pid) + '.' + input_filename_ext)
    if not os.path.isfile(input_full_name):
        raise ValueError("Could not find input: " + input_full_name)
    print("Found input: " + input_full_name)
    #
    # TODO implement conversion with XSLT file (maybe with Saxon-HE ?)


def convert_dataset_metadata_command(config, pids_file, input_dir, output_dir):
    print('Args: ' + input_dir + ',  ' + output_dir)
    print("Example using server URL: " + config['dataverse']['server_url'])

    # detect if input dir exists?
    load_path = os.path.join(config['files']['output_dir'], input_dir)

    # create output dir if not exists!
    save_path = os.path.join(config['files']['output_dir'], output_dir)
    if os.path.isdir(save_path):
        print("Skipping dir creation, because it already exists: " + save_path)
    else:
        print("Creating output dir: " + save_path)
        os.makedirs(save_path)

    # look for inputfile in configured OUTPUT_DIR
    full_name = os.path.join(config['files']['output_dir'], pids_file)
    pids = load_pids(full_name)

    batch_process(pids, lambda pid: convert_dataset_metadata_action(config['dataverse']['server_url'], pid, load_path,
                                                                    save_path), config['files']['output_dir'],
                  delay=0.0)


def main():
    config = init()
    parser = argparse.ArgumentParser(
        description='Retrieves the metadata for all published datasets with the pids in the given inputfile')
    parser.add_argument('-p', '--pids_file', default='dataset_pids.txt', help='The input file with the dataset pids')
    parser.add_argument('-i', '--input_dir', default='dataset_metadata',
                        help='The input dir with the dataset metadata files')
    parser.add_argument('-o', '--output_dir', default='converted_dataset_metadata',
                        help='The output dir, for storing the converted metadata files')
    args = parser.parse_args()

    convert_dataset_metadata_command(config, args.pids_file, args.input_dir, args.output_dir)


if __name__ == '__main__':
    main()



import argparse

import requests, logging

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids


def modify_registration_metadata(dataverse_url, dataverse_api_token, pid):
    logging.debug("Calling modifyRegistrationMetadata for {}".format(pid))
    url = '%s/api/datasets/:persistentId/modifyRegistrationMetadata?persistentId=%s' % (
        dataverse_url, pid)
    response = requests.post(
        url=url,
        headers={
            'X-Dataverse-key': dataverse_api_token
        }
    )
    logging.debug("Dataverse response: {}".format(response.text))


def update_datacite_record(dataverse_url, dataverse_api_token):
    def update_datacite_record_for_pid(pid):
        modify_registration_metadata(dataverse_url, dataverse_api_token, pid)
        return True

    return update_datacite_record_for_pid


def update_datacite_records(dataverse_url, dataverse_api_token, pid_file, out_file, delay):
    pids = load_pids(pid_file)
    batch_process(pids, update_datacite_record(dataverse_url, dataverse_api_token), out_file, delay)


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Calls the Dataverse modifyRegistrationMetadata for the PIDs in the input file, with a delay'
                    + 'between calls')
    parser.add_argument('dataset_pids', help='Newline separated file with dataset PIDs')
    parser.add_argument('processed_pids', help="Datasets that were successfully processed",
                        default='datacite-processed.txt')
    parser.add_argument('delay', help='Delay between calls', default=0.25)
    args = parser.parse_args()

    dataverse_url = config['dataverse']['server_url']
    dataverse_api_token = config['dataverse']['api_token']

    update_datacite_records(dataverse_url, dataverse_api_token, args.dataset_pids, args.processed_pids, args.delay)


if __name__ == '__main__':
    main()



import argparse
import logging

from datastation.config import init
from datastation.ds_pidsfile import store_pids
from datastation.dv_search import get_dataset_pids_from_search


def retrieve_dataset_pids_command(config, dataverse_alias, output_filename):
    logging.info("Retrieving dataset PIDs from URL {}, dataverse {}".format(config['dataverse']['server_url'], dataverse_alias))
    pids = get_dataset_pids_from_search(config['dataverse']['server_url'], dataverse_alias)
    # store in work dir, for further processing and or inspection
    store_pids(pids, config['files']['output_dir'], output_filename)


def main():
    config = init()

    parser = argparse.ArgumentParser(
        description='Retrieves the pids for all published datasets in the given collection')
    parser.add_argument('-o', '--output', help='The output file, for storing the pids retrieved')
    parser.add_argument('dataverse_alias', help='The short name (or alias) of the dataverse (collection)')
    args = parser.parse_args()

    retrieve_dataset_pids_command(config, args.dataverse_alias, args.output)


if __name__ == '__main__':
    main()



import argparse
import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import publish_dataset


def publish_dataset_command(config, server_url, api_token, pids_file, type):
    pids = load_pids(pids_file)

    # Long delay because publish is doing a lot after the async. request is returning
    # and sometimes datasets get locked
    batch_process(pids, lambda pid: publish_dataset(server_url, api_token, pid, type), config['files']['output_dir'],
                  delay=5.0)


def main():
    config = init()
    parser = argparse.ArgumentParser(description='Publishes datasets with the pids in the given inputfile')
    parser.add_argument('-p', '--pids_file', default='dataset_pids.txt', help='The input file with the dataset pids')
    parser.add_argument('-t', '--type', default='major',
                        help='The type of version upgrade, minor for metadata changes, otherwise major.')
    args = parser.parse_args()

    server_url = config['dataverse']['server_url']
    api_token = config['dataverse']['api_token']

    publish_dataset_command(config, server_url, api_token, args.pids_file, args.type)


if __name__ == '__main__':
    main()



import argparse
import os

from datastation.batch_processing import batch_process
from datastation.config import init
from datastation.ds_pidsfile import load_pids
from datastation.dv_api import delete_dataset_role_assignment, get_dataset_roleassigments


def delete_roleassignment_action(server_url, api_token, pid, role_assignee, role_alias):
    deleted_role = False
    resp_data = get_dataset_roleassigments(server_url, api_token, pid)
    # print(json.dumps(resp_data, indent=2))
    for role_assignment in resp_data:
        assignee = role_assignment['assignee']
        # role_id = role_assignment['roleId']
        alias = role_assignment['_roleAlias']
        print("Role assignee: " + assignee + ', role alias: ' + alias)
        if assignee == role_assignee and alias == role_alias:
            # delete this one
            assignment_id = role_assignment['id']
            print("Try deleting the role assignment")
            delete_dataset_role_assignment(server_url, api_token, pid, assignment_id)
            print("Done")
            deleted_role = True
        else:
            print("Leave as-is")
    return deleted_role


def delete_roleassignments_command(config, pids_file, role_assignee, role_alias):
    # look for inputfile in configured OUTPUT_DIR
    full_name = os.path.join(config['files']['output_dir'], pids_file)
    pids = load_pids(full_name)

    # could be fast, but depends on number of files inside the dataset
    batch_process(pids, lambda pid: delete_roleassignment_action(config['dataverse']['server_url'],
                                                                 config.DATAVERSE_API_TOKEN, pid, role_assignee,
                                                                 role_alias), config['files']['output_dir'], delay=1.5)


def main():
    config = init()
    parser = argparse.ArgumentParser(
        description='Delete role assigment for datasets with the pids in the given inputfile')
    parser.add_argument("role_assignee", help="Role assignee (example: @dataverseAdmin)")
    parser.add_argument("role_alias", help="Role alias (example: contributor")
    parser.add_argument('-p', '--pids_file', default='dataset_pids.txt', help='The input file with the dataset pids')
    args = parser.parse_args()

    role_assignee = args.role_assignee
    role_alias = args.role_alias

    delete_roleassignments_command(config, args.pids_file, role_assignee, role_alias)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['datastation', 'datastation.scripts']

package_data = \
{'': ['*']}

install_requires = \
['dicttoxml>=1.7.4,<2.0.0',
 'lxml>=4.8.0,<5.0.0',
 'psycopg>=3.0.15,<4.0.0',
 'pyYAML>=6.0,<7.0',
 'requests>=2.26.0,<3.0.0']

entry_points = \
{'console_scripts': ['dv-fix-provenance-files = '
                     'datastation.scripts.fix_provenance_files:main',
                     'update-datacite-records = '
                     'datastation.scripts.update_datacite_records:main']}

setup_kwargs = {
    'name': 'dans-datastation-tools',
    'version': '0.5.0',
    'description': 'Command line utilities for Data Station application management',
    'long_description': None,
    'author': 'DANS-KNAW',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime
import time
import logging


def batch_process(pids, process_action_func, output_file=None, delay=0.1, fail_on_first_error=True):
    """

    """
    if output_file is None:
        timestamp_str = '_' + datetime.now().strftime("%Y%m%d_%H%M%S")
        output_file = "out-{}.txt".format(timestamp_str)
    with open(output_file, 'w') as mutated_dataset_pids_file:
        num_pids = len(pids)
        logging.info("Start batch processing on {} datasets".format(num_pids))
        num = 0
        for pid in pids:
            num += 1
            logging.info("[{} of {}] Processing dataset with pid: {}".format(num, num_pids, pid))
            try:
                mutated = process_action_func(pid)
                if mutated:
                    mutated_dataset_pids_file.write(pid + '\n')
                    mutated_dataset_pids_file.flush()
            except Exception as e:
                logging.exception("Exception ocurred", exc_info=True)
                if fail_on_first_error:
                    logging.error("Stop processing because of an exception:  {}".format(e))
                    break
                logging.debug("fail_on_first_error is False, continuing...")
            if delay > 0 and num < num_pids:
                logging.debug("Sleeping for {} seconds...".format(delay))
                time.sleep(delay)
        logging.info("Done processing {} datasets".format(num_pids))
